/**
 * @file
 * Defines the simpleFavs Behaviour.
 */

(function ($, Drupal) {
  Drupal.behaviors.simpleFavs = {
    /**
     * Adds to the favourites storage the given contentId.
     *
     * @param contentId
     *   The content id to be stored as favourite.
     */
    addToFavs: function (contentId) {
      const favs = Drupal.behaviors.simpleFavs.getFavs();
      favs[contentId] = contentId;
      Drupal.behaviors.simpleFavs.setFavs(favs);
    },

    /**
     * Removes from the favourites storage the given contentId.
     *
     * @param contentId
     *   The content id to be removed as favourite.
     */
    removeFromFavs: function (contentId) {
      const favs = Drupal.behaviors.simpleFavs.getFavs();
      delete favs[contentId];
      Drupal.behaviors.simpleFavs.setFavs(favs);
    },

    /**
     * Sets the favs cookie.
     */
    setCookie: function (cookieValue) {
      const cookieExpires = Drupal.behaviors.simpleFavs.getExpirationDate();
      document.cookie = 'content_favs=' + JSON.stringify(cookieValue) + ';expires=' + cookieExpires.toGMTString() + ';path=/';
    },

    /**
     * Defines the cookie expiration time based on the configuration.
     *
     * @returns {Date}
     *   The expiration date for the cookie.
     */
    getExpirationDate: function () {
      const now = new Date();
      const cookieLifetime = parseInt(drupalSettings.simple_favs.heart.cookie_expiration);
      now.setDate(now.getDate() + cookieLifetime);
      return now;
    },

    /**
     * Retrieves the list of stored favourites.
     */
    getFavs: function () {
      const useDatabase = drupalSettings.simple_favs.heart.use_database_storage;
      const uid = drupalSettings.simple_favs.heart.current_uid;
      console.log('Storage mode:', useDatabase && uid !== 0 ? 'DB' : 'cookie');

      if (useDatabase && uid !== 0) {
        let favs = {};
        $.ajax({
          url: '/simple-favs/get-user-favs',
          method: 'GET',
          async: false,
          success: function (response) {
            // Defensive: ensure response is a plain object
            if (typeof response === 'object' && !Array.isArray(response)) {
              favs = response;
            }
          },
          error: function () {
            console.warn('Failed to load user favourites.');
          }
        });
        return favs;
      }

      // Fallback to cookie
      const name = "content_favs=";
      const decodedCookie = decodeURIComponent(document.cookie);
      const ca = decodedCookie.split(';');

      for (let i = 0; i < ca.length; i++) {
        let c = ca[i].trim();
        if (c.indexOf(name) === 0) {
          try {
            const parsed = JSON.parse(c.substring(name.length));
            if (typeof parsed === 'object' && !Array.isArray(parsed)) {
              return parsed;
            }
          } catch (e) {
            console.warn('Invalid cookie JSON');
          }
        }
      }

      return {};
    },

    /**
     * Writes the favourites list to DB or cookie depending on config.
     */
    setFavs: function (favs) {
      const useDatabase = drupalSettings.simple_favs.heart.use_database_storage;
      const uid = drupalSettings.simple_favs.heart.current_uid;
      console.log('Storage mode:', useDatabase && uid !== 0 ? 'DB' : 'cookie');
      console.log('setFavs:', favs);

      // Coerce into plain object if it's mistakenly an array.
      favs = Object.assign({}, favs);

      if (useDatabase && uid !== 0) {
        $.ajax({
          url: '/simple-favs/set-user-favs',
          method: 'POST',
          contentType: 'application/json',
          data: JSON.stringify(favs),
          success: function () {
            console.log('Saved favourites to database.');
          },
          error: function () {
            console.warn('Failed to save favourites to database.');
          }
        });
      }
      else {
        const cookieExpires = Drupal.behaviors.simpleFavs.getExpirationDate();
        document.cookie = 'content_favs=' + JSON.stringify(favs) +
          ';expires=' + cookieExpires.toGMTString() + ';path=/';
        console.log('Saved favourites to cookie.');
      }
    },

    /**
     * Checks if the given contentId is in favourites or not.
     *
     * @param contentId
     *   The content id to be checked.
     *
     * @return boolean
     */
    isInFavs: function (contentId) {
      const favs = Drupal.behaviors.simpleFavs.getFavs();
      return favs.hasOwnProperty(contentId);
    },
  };
})(jQuery, Drupal);

